#ifndef __GENIE_DATA_ACCESS__
#define __GENIE_DATA_ACCESS__

// GENIE data access interface in C++
//
// Designed for calling directly from FORTRAN but not
// with strings !! (we expect NULL terminated strings by
// ref at this point).
//

// I knowingly break the rule of not nesting includes wherever possible here
// this is so that a user of the interface only has one include to remeber.
// note also that GENIE I/O sabotaging is DISABLED inside geniedefs.h, just
// for this one header file.
#include "geniedefs.h"

class gdai
{

private:
    istream in;
    genie_ostream out, inf, err, dbg;
    String Format;                      // Default format for this session

public:

    // constructors
    gdai(const char *default_format, int information, int debug);
    gdai(const char *default_format,
		istream&, genie_ostream&, genie_ostream&, genie_ostream&, genie_ostream&);

    // destructor
    ~gdai();

    // Send smalltalk or GCL commands to the Open GDAI
    int gcl(const char* code);
    int smalltalk(const char* code);
    int file_format(const char* file_format);
    String file_format(void) const { return Format; }

};

//
// Interface routines are all declared so C/FORTRAN can get at them
//
typedef const char* STRING;

extern "C"
{

    // Get GENIE going
    int GX_activate_session( STRING default_format, int inf, int debug );
    // and the other thing
    int GX_deactivate_session( void );

    // for file association and contents inspection
    int GX_select_source( STRING filename );
    int GX_select_destination( STRING filename, STRING file_format );
    int GX_directory( STRING filename );

    // Data reading/writing
    int GX_get( STRING handle, STRING tag, int object_id );
    int GX_put( STRING handle, STRING tag, int object_id, STRING flag, STRING comment );


    // Routines for manipulating handles
    int GX_assign_handle( STRING handle_lval, STRING handle_rval );
    int GX_release_handle( STRING handle );

    int GX_transfer( STRING handle, STRING direction, STRING type,
                             void* data, int length, int dims[]);
    /*

    // Some other possibilities
    // friend int GX_compress_file( const char* filename );
    */

}

// Currently there is only one sessino global to Open GENIE but this could change
// When Open GENIE is running interatively this is initialized in main().
extern gdai *This_session;

#endif // GENIE_DATA_ACCESS
